@@ -30,6 +30,7 @@
 #include "libavutil/imgutils.h"
 #include "libavutil/intreadwrite.h"
 #include "libavutil/mastering_display_metadata.h"
+#include "libavutil/mem.h"
 #include "libavutil/pixfmt.h"
 #include "libavutil/rational.h"
 #include "libavutil/stereo3d.h"
@@ -41,8 +42,8 @@
 #include "apng.h"
 #include "png.h"
 #include "pngdsp.h"
+#include "progressframe.h"
 #include "thread.h"
-#include "threadframe.h"
 #include "zlib_wrapper.h"
 
 #include <zlib.h>
@@ -62,8 +63,8 @@ typedef struct PNGDecContext {
     AVCodecContext *avctx;
 
     GetByteContext gb;
-    ThreadFrame last_picture;
-    ThreadFrame picture;
+    ProgressFrame last_picture;
+    ProgressFrame picture;
 
     AVDictionary *frame_metadata;
 
@@ -85,11 +86,12 @@ typedef struct PNGDecContext {
     int have_clli;
     uint32_t clli_max;
     uint32_t clli_avg;
-    int have_mdvc;
-    uint16_t mdvc_primaries[3][2];
-    uint16_t mdvc_white_point[2];
-    uint32_t mdvc_max_lum;
-    uint32_t mdvc_min_lum;
+    /* Mastering Display Color Volume */
+    int have_mdcv;
+    uint16_t mdcv_primaries[3][2];
+    uint16_t mdcv_white_point[2];
+    uint32_t mdcv_max_lum;
+    uint32_t mdcv_min_lum;
 
     enum PNGHeaderState hdr_state;
     enum PNGImageState pic_state;
@@ -762,24 +764,24 @@ static int populate_avctx_color_fields(AVCodecContext *avctx, AVFrame *frame)
         }
     }
 
-    if (s->have_mdvc) {
-        AVMasteringDisplayMetadata *mdvc;
+    if (s->have_mdcv) {
+        AVMasteringDisplayMetadata *mdcv;
 
-        ret = ff_decode_mastering_display_new(avctx, frame, &mdvc);
+        ret = ff_decode_mastering_display_new(avctx, frame, &mdcv);
         if (ret < 0)
             return ret;
 
-        if (mdvc) {
-            mdvc->has_primaries = 1;
+        if (mdcv) {
+            mdcv->has_primaries = 1;
             for (int i = 0; i < 3; i++) {
-                mdvc->display_primaries[i][0] = av_make_q(s->mdvc_primaries[i][0], 50000);
-                mdvc->display_primaries[i][1] = av_make_q(s->mdvc_primaries[i][1], 50000);
+                mdcv->display_primaries[i][0] = av_make_q(s->mdcv_primaries[i][0], 50000);
+                mdcv->display_primaries[i][1] = av_make_q(s->mdcv_primaries[i][1], 50000);
             }
-            mdvc->white_point[0] = av_make_q(s->mdvc_white_point[0], 50000);
-            mdvc->white_point[1] = av_make_q(s->mdvc_white_point[1], 50000);
-            mdvc->has_luminance = 1;
-            mdvc->max_luminance = av_make_q(s->mdvc_max_lum, 10000);
-            mdvc->min_luminance = av_make_q(s->mdvc_min_lum, 10000);
+            mdcv->white_point[0] = av_make_q(s->mdcv_white_point[0], 50000);
+            mdcv->white_point[1] = av_make_q(s->mdcv_white_point[1], 50000);
+            mdcv->has_luminance = 1;
+            mdcv->max_luminance = av_make_q(s->mdcv_max_lum, 10000);
+            mdcv->min_luminance = av_make_q(s->mdcv_min_lum, 10000);
         }
     }
 
@@ -873,7 +875,7 @@ static int decode_idat_chunk(AVCodecContext *avctx, PNGDecContext *s,
             s->bpp += byte_depth;
         }
 
-        ff_thread_release_ext_buffer(&s->picture);
+        ff_progress_frame_unref(&s->picture);
         if (s->dispose_op == APNG_DISPOSE_OP_PREVIOUS) {
             /* We only need a buffer for the current picture. */
             ret = ff_thread_get_buffer(avctx, p, 0);
@@ -882,8 +884,8 @@ static int decode_idat_chunk(AVCodecContext *avctx, PNGDecContext *s,
         } else if (s->dispose_op == APNG_DISPOSE_OP_BACKGROUND) {
             /* We need a buffer for the current picture as well as
              * a buffer for the reference to retain. */
-            ret = ff_thread_get_ext_buffer(avctx, &s->picture,
-                                           AV_GET_BUFFER_FLAG_REF);
+            ret = ff_progress_frame_get_buffer(avctx, &s->picture,
+                                               AV_GET_BUFFER_FLAG_REF);
             if (ret < 0)
                 return ret;
             ret = ff_thread_get_buffer(avctx, p, 0);
@@ -891,8 +893,9 @@ static int decode_idat_chunk(AVCodecContext *avctx, PNGDecContext *s,
                 return ret;
         } else {
             /* The picture output this time and the reference to retain coincide. */
-            if ((ret = ff_thread_get_ext_buffer(avctx, &s->picture,
-                                                AV_GET_BUFFER_FLAG_REF)) < 0)
+            ret = ff_progress_frame_get_buffer(avctx, &s->picture,
+                                                AV_GET_BUFFER_FLAG_REF);
+            if (ret < 0)
                 return ret;
             ret = av_frame_ref(p, s->picture.f);
             if (ret < 0)
@@ -1016,7 +1019,7 @@ static int decode_trns_chunk(AVCodecContext *avctx, PNGDecContext *s,
 
         for (i = 0; i < length / 2; i++) {
             /* only use the least significant bits */
-            v = av_mod_uintp2(bytestream2_get_be16(gb), s->bit_depth);
+            v = av_zero_extend(bytestream2_get_be16(gb), s->bit_depth);
 
             if (s->bit_depth > 8)
                 AV_WB16(&s->transparent_color_be[2 * i], v);
@@ -1081,17 +1084,20 @@ static int decode_sbit_chunk(AVCodecContext *avctx, PNGDecContext *s,
         return AVERROR_INVALIDDATA;
     }
 
-    channels = ff_png_get_nb_channels(s->color_type);
+    channels = s->color_type & PNG_COLOR_MASK_PALETTE ? 3 : ff_png_get_nb_channels(s->color_type);
 
-    if (bytestream2_get_bytes_left(gb) != channels)
+    if (bytestream2_get_bytes_left(gb) != channels) {
+        av_log(avctx, AV_LOG_ERROR, "Invalid sBIT size: %d, expected: %d\n",
+            bytestream2_get_bytes_left(gb), channels);
         return AVERROR_INVALIDDATA;
+    }
 
     for (int i = 0; i < channels; i++) {
         int b = bytestream2_get_byteu(gb);
         bits = FFMAX(b, bits);
     }
 
-    if (bits < 0 || bits > s->bit_depth) {
+    if (bits <= 0 || bits > (s->color_type & PNG_COLOR_MASK_PALETTE ? 8 : s->bit_depth)) {
         av_log(avctx, AV_LOG_ERROR, "Invalid significant bits: %d\n", bits);
         return AVERROR_INVALIDDATA;
     }
@@ -1253,7 +1259,7 @@ static void handle_p_frame_png(PNGDecContext *s, AVFrame *p)
 
     ls = FFMIN(ls, s->width * s->bpp);
 
-    ff_thread_await_progress(&s->last_picture, INT_MAX, 0);
+    ff_progress_frame_await(&s->last_picture, INT_MAX);
     for (j = 0; j < s->height; j++) {
         for (i = 0; i < ls; i++)
             pd[i] += pd_last[i];
@@ -1285,7 +1291,7 @@ static int handle_p_frame_apng(AVCodecContext *avctx, PNGDecContext *s,
         return AVERROR_PATCHWELCOME;
     }
 
-    ff_thread_await_progress(&s->last_picture, INT_MAX, 0);
+    ff_progress_frame_await(&s->last_picture, INT_MAX);
 
     // copy unchanged rectangles from the last frame
     for (y = 0; y < s->y_offset; y++)
@@ -1569,20 +1575,20 @@ static int decode_frame_common(AVCodecContext *avctx, PNGDecContext *s,
             s->clli_max = bytestream2_get_be32u(&gb_chunk);
             s->clli_avg = bytestream2_get_be32u(&gb_chunk);
             break;
-        case MKTAG('m', 'D', 'V', 'c'):
+        case MKTAG('m', 'D', 'C', 'v'):
             if (bytestream2_get_bytes_left(&gb_chunk) != 24) {
-                av_log(avctx, AV_LOG_WARNING, "Invalid mDVc chunk size: %d\n", bytestream2_get_bytes_left(&gb_chunk));
+                av_log(avctx, AV_LOG_WARNING, "Invalid mDCv chunk size: %d\n", bytestream2_get_bytes_left(&gb_chunk));
                 break;
             }
-            s->have_mdvc = 1;
+            s->have_mdcv = 1;
             for (int i = 0; i < 3; i++) {
-                s->mdvc_primaries[i][0] = bytestream2_get_be16u(&gb_chunk);
-                s->mdvc_primaries[i][1] = bytestream2_get_be16u(&gb_chunk);
+                s->mdcv_primaries[i][0] = bytestream2_get_be16u(&gb_chunk);
+                s->mdcv_primaries[i][1] = bytestream2_get_be16u(&gb_chunk);
             }
-            s->mdvc_white_point[0] = bytestream2_get_be16u(&gb_chunk);
-            s->mdvc_white_point[1] = bytestream2_get_be16u(&gb_chunk);
-            s->mdvc_max_lum = bytestream2_get_be32u(&gb_chunk);
-            s->mdvc_min_lum = bytestream2_get_be32u(&gb_chunk);
+            s->mdcv_white_point[0] = bytestream2_get_be16u(&gb_chunk);
+            s->mdcv_white_point[1] = bytestream2_get_be16u(&gb_chunk);
+            s->mdcv_max_lum = bytestream2_get_be32u(&gb_chunk);
+            s->mdcv_min_lum = bytestream2_get_be32u(&gb_chunk);
             break;
         case MKTAG('I', 'E', 'N', 'D'):
             if (!(s->pic_state & PNG_ALLIMAGE))
@@ -1673,7 +1679,7 @@ exit_loop:
     }
 
     /* handle P-frames only if a predecessor frame is available */
-    if (s->last_picture.f->data[0]) {
+    if (s->last_picture.f) {
         if (   !(avpkt->flags & AV_PKT_FLAG_KEY) && avctx->codec_tag != AV_RL32("MPNG")
             && s->last_picture.f->width == p->width
             && s->last_picture.f->height== p->height
@@ -1690,12 +1696,11 @@ exit_loop:
     if (CONFIG_APNG_DECODER && s->dispose_op == APNG_DISPOSE_OP_BACKGROUND)
         apng_reset_background(s, p);
 
-    ff_thread_report_progress(&s->picture, INT_MAX, 0);
-
-    return 0;
-
+    ret = 0;
 fail:
-    ff_thread_report_progress(&s->picture, INT_MAX, 0);
+    if (s->picture.f)
+        ff_progress_frame_report(&s->picture, INT_MAX);
+
     return ret;
 }
 
@@ -1782,8 +1787,8 @@ static int decode_frame_png(AVCodecContext *avctx, AVFrame *p,
         goto the_end;
 
     if (!(avctx->active_thread_type & FF_THREAD_FRAME)) {
-        ff_thread_release_ext_buffer(&s->last_picture);
-        FFSWAP(ThreadFrame, s->picture, s->last_picture);
+        ff_progress_frame_unref(&s->last_picture);
+        FFSWAP(ProgressFrame, s->picture, s->last_picture);
     }
 
     *got_frame = 1;
@@ -1834,12 +1839,9 @@ static int decode_frame_apng(AVCodecContext *avctx, AVFrame *p,
         return ret;
 
     if (!(avctx->active_thread_type & FF_THREAD_FRAME)) {
-        if (s->dispose_op == APNG_DISPOSE_OP_PREVIOUS) {
-            ff_thread_release_ext_buffer(&s->picture);
-        } else {
-            ff_thread_release_ext_buffer(&s->last_picture);
-            FFSWAP(ThreadFrame, s->picture, s->last_picture);
-        }
+        if (s->dispose_op != APNG_DISPOSE_OP_PREVIOUS)
+            FFSWAP(ProgressFrame, s->picture, s->last_picture);
+        ff_progress_frame_unref(&s->picture);
     }
 
     *got_frame = 1;
@@ -1852,8 +1854,7 @@ static int update_thread_context(AVCodecContext *dst, const AVCodecContext *src)
 {
     PNGDecContext *psrc = src->priv_data;
     PNGDecContext *pdst = dst->priv_data;
-    ThreadFrame *src_frame = NULL;
-    int ret;
+    const ProgressFrame *src_frame;
 
     if (dst == src)
         return 0;
@@ -1878,12 +1879,7 @@ static int update_thread_context(AVCodecContext *dst, const AVCodecContext *src)
     src_frame = psrc->dispose_op == APNG_DISPOSE_OP_PREVIOUS ?
                 &psrc->last_picture : &psrc->picture;
 
-    ff_thread_release_ext_buffer(&pdst->last_picture);
-    if (src_frame && src_frame->f->data[0]) {
-        ret = ff_thread_ref_frame(&pdst->last_picture, src_frame);
-        if (ret < 0)
-            return ret;
-    }
+    ff_progress_frame_replace(&pdst->last_picture, src_frame);
 
     return 0;
 }
@@ -1894,10 +1890,6 @@ static av_cold int png_dec_init(AVCodecContext *avctx)
     PNGDecContext *s = avctx->priv_data;
 
     s->avctx = avctx;
-    s->last_picture.f = av_frame_alloc();
-    s->picture.f = av_frame_alloc();
-    if (!s->last_picture.f || !s->picture.f)
-        return AVERROR(ENOMEM);
 
     ff_pngdsp_init(&s->dsp);
 
@@ -1908,10 +1900,8 @@ static av_cold int png_dec_end(AVCodecContext *avctx)
 {
     PNGDecContext *s = avctx->priv_data;
 
-    ff_thread_release_ext_buffer(&s->last_picture);
-    av_frame_free(&s->last_picture.f);
-    ff_thread_release_ext_buffer(&s->picture);
-    av_frame_free(&s->picture.f);
+    ff_progress_frame_unref(&s->last_picture);
+    ff_progress_frame_unref(&s->picture);
     av_freep(&s->buffer);
     s->buffer_size = 0;
     av_freep(&s->last_row);
@@ -1939,7 +1929,7 @@ const FFCodec ff_apng_decoder = {
     UPDATE_THREAD_CONTEXT(update_thread_context),
     .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_FRAME_THREADS,
     .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP |
-                      FF_CODEC_CAP_ALLOCATE_PROGRESS |
+                      FF_CODEC_CAP_USES_PROGRESSFRAMES |
                       FF_CODEC_CAP_ICC_PROFILES,
 };
 #endif
@@ -1957,7 +1947,8 @@ const FFCodec ff_png_decoder = {
     UPDATE_THREAD_CONTEXT(update_thread_context),
     .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_FRAME_THREADS,
     .caps_internal  = FF_CODEC_CAP_SKIP_FRAME_FILL_PARAM |
-                      FF_CODEC_CAP_ALLOCATE_PROGRESS | FF_CODEC_CAP_INIT_CLEANUP |
+                      FF_CODEC_CAP_INIT_CLEANUP |
+                      FF_CODEC_CAP_USES_PROGRESSFRAMES |
                       FF_CODEC_CAP_ICC_PROFILES,
 };
 #endif
